/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include "nrf.h"
#include <stdbool.h>
#include <stdint.h>
#include "bsp.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "math.h"

// Peripheral channel assignments
//#define PWM0_GPIOTE_CH      0
//#define PWM0_PPI_CH_A       0
#define PWM0_TIMER_CC_NUM   0


// TIMER3 reload value. The PWM frequency equals '16000000 / TIMER_RELOAD'
#define TIMER_RELOAD        1000
// The timer CC register used to reset the timer. Be aware that not all timers in the nRF52 have 6 CC registers.
#define TIMER_RELOAD_CC_NUM 0

#define PIN                 30


// This function initializes timer 3 with the following configuration:
// 24-bit, base frequency 16 MHz, auto clear on COMPARE5 match (CC5 = TIMER_RELOAD)
//void timer_init()
//{
//    NRF_TIMER3->BITMODE                 = TIMER_BITMODE_BITMODE_24Bit << TIMER_BITMODE_BITMODE_Pos;
//    NRF_TIMER3->PRESCALER               = 0;
//    NRF_TIMER3->SHORTS                  = TIMER_SHORTS_COMPARE0_CLEAR_Msk << TIMER_RELOAD_CC_NUM;
//    NRF_TIMER3->MODE                    = TIMER_MODE_MODE_Timer << TIMER_MODE_MODE_Pos;
//    NRF_TIMER3->CC[TIMER_RELOAD_CC_NUM] = TIMER_RELOAD;    
//}
void timer_init()
{
    NRF_TIMER1->BITMODE                 = TIMER_BITMODE_BITMODE_16Bit << TIMER_BITMODE_BITMODE_Pos;     // 0<<0; = 0 
    NRF_TIMER1->PRESCALER               = 4;                                                            // previously 0;
    NRF_TIMER1->SHORTS                  = TIMER_SHORTS_COMPARE0_CLEAR_Msk << TIMER_RELOAD_CC_NUM;       // 1<<0; = 1
    NRF_TIMER1->MODE                    = TIMER_MODE_MODE_Timer << TIMER_MODE_MODE_Pos;                 // 0<<0; = 0
    NRF_TIMER1->CC[TIMER_RELOAD_CC_NUM] = TIMER_RELOAD;                                                 // 1000;
}


//// Starts TIMER3
//void timer_start()
//{
//    NRF_TIMER3->TASKS_START = 1;
//}

// Starts TIMER1
void timer_start()
{
    NRF_TIMER1->TASKS_START = 1;
}


// This function sets up TIMER3, the PPI and the GPIOTE modules to configure a single PWM channel
// Timer CC num, PPI channel nums and GPIOTE channel num is defined at the top of this file

void pwm0_init(uint32_t pinselect)
{  
    NRF_GPIOTE->CONFIG[0] = GPIOTE_CONFIG_MODE_Task << GPIOTE_CONFIG_MODE_Pos |                             // 3<<0
                                         GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos |      // 3<<16
                                         PIN << GPIOTE_CONFIG_PSEL_Pos |                                    // 30<<8
                                         GPIOTE_CONFIG_OUTINIT_High << GPIOTE_CONFIG_OUTINIT_Pos;           // 1<<20

    NRF_PPI->CH[0].EEP = (uint32_t)&NRF_TIMER1->EVENTS_COMPARE[TIMER_RELOAD_CC_NUM];
    NRF_PPI->CH[0].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[0];
    
    NRF_PPI->CHENSET               = (1 << 0);
}

//void pwm0_init(uint32_t pinselect)
//{  
//    NRF_GPIOTE->CONFIG[PWM0_GPIOTE_CH] = GPIOTE_CONFIG_MODE_Task << GPIOTE_CONFIG_MODE_Pos | 
//                                         GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos | 
//                                         pinselect << GPIOTE_CONFIG_PSEL_Pos | 
//                                         GPIOTE_CONFIG_OUTINIT_High << GPIOTE_CONFIG_OUTINIT_Pos;

//    NRF_PPI->CH[PWM0_PPI_CH_A].EEP = (uint32_t)&NRF_TIMER1->EVENTS_COMPARE[TIMER_RELOAD_CC_NUM];
//    NRF_PPI->CH[PWM0_PPI_CH_A].TEP = (uint32_t)&NRF_GPIOTE->TASKS_OUT[PWM0_GPIOTE_CH];
//    
//    NRF_PPI->CHENSET               = (1 << PWM0_PPI_CH_A);
//}



// Utility function for providing sin values, and converting them to integers.
// input values in the range [0 - input_max] will be converted to 0-360 degrees (0-2*PI).
// output values will be scaled to the range [output_min - output_max].



int main(void)
{
    // Initialize the timer
    timer_init();
    
    // Initialize PWM channel
    pwm0_init(LED_1);
        
    // Start the timer
    timer_start();

    while (true)
    {

    }
}
